--[[--------------------------------------------------------------------------------------------------
SpriteSheetHandler

Version: 		007
Author: 		Torben Ratzlaff
Website: 		screenheadmedia.com

Purpose:
Maintains and gives global access to all spritesheets.

----------------------------------------------------------------------------------------------------]]


--[[--------------------------------------------------------------------------------------------------
How to use:

--loads a spritesheet with a given name from a basepath
SpriteSheetHandler.loadData(name, basePath)
--name 		- [required] name of the spritesheet
--basePath 	- [optional] path of the spritesheet, if non is probvided the default base path is used
--return	- returns the loaded spritesheet


--loads several spritesheets via SpriteSheetHandler.loadData()
SpriteSheetHandler.loadDataSet(dataNames)
--dataNames 	- [required] a table containing the names of the spritesheets you want to load (see SpriteSheetHandler.loadData() for more information)


--unloads a spritesheet with a given name. Also calls SpriteSheetHandler.unloadGraphic(name).
SpriteSheetHandler.unloadData(name) 
--name 		- [required] name of the spritesheet


--unloads several sprite sheet data
SpriteSheetHandler.unloadDataSet(dataNames)
--dataNames - [required] a table containing the names of the sprite sheets you want to unload. Check SpriteSheetHandler.unloadData() for more inforation.


--creates an image sheet
SpriteSheetHandler.loadGraphic(name, basePath) 
--name 		- name of the image and the previously loaded spritesheet (have to be identical)
--basePath 	- path of the image, if non is probvided the default base path is used
--return 	- returns the created imageSheet


--loads several spritesheets via SpriteSheetHandler.loadData()
SpriteSheetHandler.loadGraphicSet(dataNames)
--dataSet 	- [required] a table containing the names of the graphics you want to load (see SpriteSheetHandler.loadGraphic() for more information)


--unloads an image sheet
SpriteSheetHandler.unloadGraphic(name) 
--name 		- name of the imagesheet


--unloads several sprite sheet data
SpriteSheetHandler.unloadGraphicSet(dataNames)
--dataNames - [required] a table containing the names of the graphics you want to unload.(Check SpriteSheetHandler.unloadGraphic() for more inforation.)


SpriteSheetHandler.unloadAllData() --unloads all image sheets and sprite sheets


SpriteSheetHandler.getSheetData(name, basePath) --returns a loaded sprite sheet or loads it, if it hasn't been loaded already
--name 		- [required] name of the spritesheet
--basePath 	- [optional] path of the spritesheet, if non is probvided the default base path is used
--return	- returns the spritesheet


SpriteSheetHandler.getImageSheet(name, basePath) --returns a loaded image sheet or loads it, if it hasn't been loaded already
--name 		- [required] name of the image and the previously loaded spritesheet (have to be identical)
--basePath 	- [optional] path of the image, if non is probvided the default base path is used
--return 	- returns the imageSheet


SpriteSheetHandler.getFrameIndex(name, basePath) --returns the frameIndex of a loaded sprite sheet or loads it, if it hasn't been loaded already 
--name 		- [required] name of the spritesheet
--basePath 	- [optional] path of the spritesheet, if non is probvided the default base path is used
--return	- returns the frameIndex


SpriteSheetHandler.setDataPath(path) --sets the default path to load spritesheets from
--path 		- [required] path of the spritesheets (default is "spritedata.")


SpriteSheetHandler.setImagePath(path) --sets the default path to load images from
--basePath 	- [required] path of the images (default is "images/")

----------------------------------------------------------------------------------------------------]]


local graphics_newImageSheet = graphics.newImageSheet
local display_remove = display.remove
local package_loaded = package.loaded
local pairs = pairs


local imageBasePath = "images/"
local dataBasePath 	= "spritedata."

local SpriteSheetHandler = {}

local allSheetData = {}
SpriteSheetHandler.sheets = allSheetData


function SpriteSheetHandler.loadData(name, basePath)
	if not allSheetData[name] then
		local basePath 		= basePath or dataBasePath
		local loadedData 	= require(basePath..name)

		local sheetData = {}
		sheetData.data 			= loadedData
		sheetData.frameIndex 	= loadedData.frameIndex

		allSheetData[name] = sheetData

		return loadedData
	else
		return allSheetData[name]
	end
end


local SpriteSheetHandler_loadData = SpriteSheetHandler.loadData
function SpriteSheetHandler.loadDataSet(dataNames)
	for i=1, #dataNames do
		SpriteSheetHandler_loadData(dataNames[i])
	end
end


function SpriteSheetHandler.unloadData(name)
	local sheetData = allSheetData[name]
	
	if sheetData then
		SpriteSheetHandler.unloadGraphic(name)

		sheetData.data 			= nil
		sheetData.frameIndex 	= nil
		allSheetData[name] 		= nil
		package_loaded[name] 	= nil
	end
end


local SpriteSheetHandler_unloadData = SpriteSheetHandler.unloadData
function SpriteSheetHandler.unloadDataSet(dataNames)
	for i=1, #dataNames do
		SpriteSheetHandler_unloadData(dataNames[i])
	end
end


function SpriteSheetHandler.loadGraphic(name, basePath)
	local sheetData = allSheetData[name]

	if not sheetData then
		sheetData = SpriteSheetHandler.loadData(name)
	end

	if not sheetData.imageSheet then
		local basePath 			= basePath or imageBasePath
		local imageSheet 		= graphics_newImageSheet(basePath..name..".png", sheetData.data:getSheet())
		sheetData.imageSheet 	= imageSheet

		return imageSheet
	else
		return sheetData.imageSheet
	end
end


local SpriteSheetHandler_loadGraphic = SpriteSheetHandler.loadGraphic
function SpriteSheetHandler.loadGraphicSet(dataNames)
	for i=1, #dataNames do
		SpriteSheetHandler_loadGraphic(dataNames[i])
	end
end


function SpriteSheetHandler.unloadGraphic(name)
	local sheetData = allSheetData[name]
	
	if sheetData then
		display_remove(sheetData.imageSheet)
		sheetData.imageSheet = nil
	end
end


local SpriteSheetHandler_unloadGraphic = SpriteSheetHandler.unloadGraphic
function SpriteSheetHandler.unloadGraphicSet(dataNames)
	for i=1, #dataNames do
		SpriteSheetHandler_unloadGraphic(dataNames[i])
	end
end


function SpriteSheetHandler.unloadAllData()
	for key, value in pairs(allSheetData) do
		SpriteSheetHandler.unloadData(key)
	end
	
	allSheetData = {}
end


function SpriteSheetHandler.getSheetData(name, basePath)
	local sheetData = allSheetData[name]

	if not sheetData then
		return SpriteSheetHandler.loadData(name, basePath)
	end

	return sheetData
end


function SpriteSheetHandler.getImageSheet(name, basePath)
	local sheetData = allSheetData[name]

	if not sheetData then
		sheetData = SpriteSheetHandler.loadData(name)
	end

	if not sheetData.imageSheet then
		return SpriteSheetHandler.loadGraphic(name, basePath)
	end

	return sheetData.imageSheet
end


function SpriteSheetHandler.getFrameIndex(name, basePath)
	local sheetData = allSheetData[name]

	if not sheetData then
		return SpriteSheetHandler.loadData(name, basePath).frameIndex
	end

	return sheetData.frameIndex
end


function SpriteSheetHandler.setDataPath(path)
	dataBasePath = path
end


function SpriteSheetHandler.setImagePath(path)
	imageBasePath = path
end


return SpriteSheetHandler